comment %
These macros are adaptations taken from Agner Fog's file, TSCTestB32.asm:
	 2005-07-16 Agner Fog
	 2004 GNU General Public License www.gnu.org/copyleft/gpl.html

"....  the GNU General Public License is intended to guarantee your freedom to share 
and change free software..." 

The original code was written for C++ compiling.  The only modifications, aside from
a separation into two macros, involve conversion of a procedure into inline code for 
use with MASM32.  Note that the registers ebp and edi should not be altered by code
being tested.
%
;************************************************************************************
agner_begin MACRO	count_data:REQ
LOCAL	CLOOP2

; ---------------------- DATA SEGMENT ----------------------
IFNDEF __agnermacros
	__agnermacros	equ <1>

.data
ALIGN 8
	overhead qword   ?                ; timing overhead
	clocks   dword   ?                ; pointer to results array
ENDIF

; ---------------------- code segment ----------------------
.code
        push    ebx
        push    esi
        push    edi
        push    ebp

        ; pointer to results list
        mov     esi, count_data
        mov     clocks, esi

        ; get list into cache
        cld
        mov     ecx, NUMTESTS * 2
        rep     lodsd

        ; dummy test loop without test code to measure overhead 
        xor     ebp, ebp
        mov     edi, clocks
        
@@:     sub     eax, eax
        cpuid                     ; serialize
        rdtsc                     ; read time stamp counter into edx:eax
        mov     [edi+ebp], eax    ; save in list
        mov     [edi+ebp+4], edx  ; save high dword in list
        sub     eax, eax
        cpuid                     ; serialize again
        ; void to measure overhead only

        sub     eax, eax
        cpuid                     ; serialize
        rdtsc                     ; read time stamp counter into edx:eax
        sub     eax, [edi+ebp]    ; subtract old value
        sbb     edx, [edi+ebp+4]
        mov     [edi+ebp], eax    ; save difference in list
        mov     [edi+ebp+4], edx
        sub     eax, eax           ;no effect on P4
        cpuid                     ; serialize again

        add     ebp, 8           
        cmp     ebp, NUMTESTS*8
        jb      @B	            ; dummy loop end

        ; find smallest clock count:
        mov     edi, clocks
        mov     eax, -1
        cdq
        xor     ebx, ebx
        xor     ecx, ecx
IF NUMTESTS GT 1
        mov     [edi], eax        ; exclude first count from minimum
        mov     [edi+4], edx
ENDIF
        xor     ebp, ebp

; (If cpu doesn't support conditional moves then replace all cmov by conditional jumps)
@@:        cmp     [edi+ebp],eax     ; compare low dword and get result into bl
        setb    bl
        cmp     [edi+ebp+4],edx   ; compare high dword and get result into cl
        setb    cl
        cmove   ecx, ebx          ; replace cl by bl if high dword equal
        test    cl, cl
        cmovnz  eax, [edi+ebp]    ; get lowest qword into edx:eax
        cmovnz  edx, [edi+ebp+4]
        add     ebp, 8
        cmp     ebp, NUMTESTS*8
        jb      @B		;MINLOOP
        mov     dword ptr overhead, eax  ; save minimum overhead
        mov     dword ptr overhead+4, edx

; loop to measure clock cycles of test code
        xor     ebp, ebp
        mov     edi, clocks
CLOOP2:
__CLOOP2 equ <CLOOP2>
        sub     eax, eax
        cpuid                     ; serialize
        rdtsc                     ; read time stamp counter into edx:eax
        mov     [edi+ebp], eax    ; save in list
        mov     [edi+ebp+4], edx  ; save high dword in list
        sub     eax, eax
        cpuid                     ; serialize again -248 clocks
ENDM	;Agner_begin

; ###################### Test code here ######################
;
; put your test code here. Example:
;
;        REPT 100          ; repeat code 100 times
;        shr eax, 5
;        ENDM
;
; ###################### Test code end  ######################

agner_finish MACRO	; loopcount:REQ, skips:REQ, priority

       sub     eax, eax
       cpuid                     ; serialize
        rdtsc                     ; read time stamp counter into edx:eax
        sub     eax, [edi+ebp]    ; subtract old value
        sbb     edx, [edi+ebp+4]
        mov     [edi+ebp], eax    ; save difference in list
        mov     [edi+ebp+4], edx
        sub     eax, eax           ;No effect on P4
        cpuid                     ; serialize again

        add     ebp, 8
        cmp     ebp, NUMTESTS*8
        jb      __CLOOP2

        ; subtract overhead from clock counts:
        xor     ebp, ebp
        mov     edi, clocks
        mov     eax, dword ptr overhead
        mov     edx, dword ptr overhead + 4

@@:     sub     [edi+ebp], eax
        sbb     [edi+ebp+4], edx
        add     ebp, 8
        cmp     ebp, NUMTESTS*8
        jb      @B

        mov     eax, NUMTESTS
        pop     ebp
        pop     edi
        pop     esi
        pop     ebx
ENDM	;agner_finish
